const AppSys       = imports.gi.Shell.AppSystem.get_default();
const GMenu        = imports.gi.GMenu;
const AppFavorites = imports.ui.appFavorites;
const Util         = imports.misc.util;
const ViewSelector = imports.ui.main.overview._viewSelector;
const AppDisplay   = imports.ui.appDisplay;
const Me           = imports.misc.extensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const Prefs       = Me.imports.prefs;
const UserMenu    = Me.imports.userMenu;

const SEARCH_CATEGORY = 'b88f323f-bff3-4106-8c21-de6974f38ede';

let providers  = [];

function init() {
    let categories = [];

    let hidden = [];
    let entries = Prefs.getHiddenApps().concat(Prefs.getMovedApps());
    for (let i = 0; i < entries.length; ++i){
	    hidden[i] = AppSys.lookup_app(entries[i]);
    }

    let updateFor = function() {
        let jindex = function() {
            let menus = Prefs.getMenus();
            let x = -1;
            return function(dir) {
                for (let i = 0; i < menus.length; ++i) {
                    for (let j = 0; j< menus[i].length; ++j) {
                        if (menus[i][j] == dir.get_menu_id()) {
                            return [i, j];
                        }
                    }
                }
                return [menus.length, ++x];
            };
        }();

        return function (dir) {
            let [i, j] = jindex(dir);

            if (categories[i] == null) {
                categories[i] = {
                    apps     : [],
                    names    : [],
                    found    : []
                };
            }
            categories[i].names[j] = dir.get_name();
            categories[i].found[j] = false;

            return function(app) {
                if (hidden.indexOf(app) >= 0 ) {
                    return;
                }
                categories[i].found[j] = true;
		        Util.insertSorted(categories[i].apps, app, function (a, b) {
			        let aFav = AppFavorites.getAppFavorites().isFavorite(a.get_id());
			        let bFav = AppFavorites.getAppFavorites().isFavorite(b.get_id());
                    
			        return aFav != bFav ? (aFav ? -1 : 1) : a.compare_by_name(b);
		        });
            };
        }
    }();

    let dIter = AppSys.get_tree().get_root_directory().iter();
    for (let dNext = dIter.next(); dNext != GMenu.TreeItemType.INVALID; dNext = dIter.next()) {
	    if (dNext == GMenu.TreeItemType.DIRECTORY) {
	        let dir = dIter.get_directory();
	        if (dir.get_is_nodisplay()) {
		        continue;
	        }
	        let updateWith = updateFor(dir);
	        let aIter = dir.iter();
	        let aNext = {};
	        for (let aNext = aIter.next(); aNext != GMenu.TreeItemType.INVALID; aNext = aIter.next()) {
		        if (aNext == GMenu.TreeItemType.ENTRY) {
		            let app = AppSys.lookup_app_by_tree_entry(aIter.get_entry());
		            if (app.get_app_info().get_nodisplay()) {
			            continue;
		            }
                    updateWith(app);
		        }
	        }
	    }
    }

    for (let i = 0, x = 0; i < categories.length; ++i) {
        if (typeof categories[i] == 'undefined') {
            ++x;
            continue;
        }
        let title = categories[i].names.filter(function(e, j, a) { 
            return categories[i].found[j];
        }).join(', ');

        let c = title.lastIndexOf(',');

        if (c > 0 && c < title.length - 2 && title.indexOf('&') < 0) {
            title = title.substring(0, c) + ' &' + title.substring(c + 1);
        }

        let p = new AppDisplay.AppSearchProvider();
        let a = categories[i].apps;

        p.title = title;
        p.getSubsearchResultSet = function(previousResults, terms) {
            p.searchSystem.pushResults(p, []);
        };
        p.getInitialResultSet = function(terms) {
	        if (terms == SEARCH_CATEGORY) {
                p.searchSystem.pushResults(p, a);
	        } else {
	            p.searchSystem.pushResults(p, []);
	        }
        };
        providers[i - x] = p;
    }
}

function enable() {
    for (let i = 0; i < providers.length; i++) {
	    ViewSelector.addSearchProvider(providers[i]);
    }
}

function disable() {
    for (let i = 0; i < providers.length; i++) {
	    ViewSelector.removeSearchProvider(providers[i]);
    }
}
